/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.basic.fragment;


import android.net.Uri;

import androidx.fragment.app.FragmentActivity;

import com.compdfkit.core.annotation.CPDFLinkAnnotation;
import com.compdfkit.core.annotation.CPDFTextAnnotation;
import com.compdfkit.core.annotation.form.CPDFComboboxWidget;
import com.compdfkit.core.annotation.form.CPDFListboxWidget;
import com.compdfkit.core.annotation.form.CPDFSignatureWidget;
import com.compdfkit.core.edit.CPDFEditPage;
import com.compdfkit.tools.R;
import com.compdfkit.tools.annotation.pdfproperties.pdflink.CLinkAnnotAttachHelper;
import com.compdfkit.tools.annotation.pdfproperties.pdfnote.CPDFtextAnnotAttachHelper;
import com.compdfkit.tools.annotation.pdfproperties.pdfnote.CPDFtextAnnotImpl;
import com.compdfkit.tools.common.contextmenu.CPDFContextMenuHelper;
import com.compdfkit.tools.common.contextmenu.impl.CEditImageContextMenuView;
import com.compdfkit.tools.common.contextmenu.impl.CEditTextContextMenuView;
import com.compdfkit.tools.common.contextmenu.impl.CScreenShotContextMenuView;
import com.compdfkit.tools.common.contextmenu.impl.CSearchReplaceContextMenuView;
import com.compdfkit.tools.common.contextmenu.impl.CSignatureContextMenuView;
import com.compdfkit.tools.common.pdf.config.CPDFConfiguration;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.dialog.CLoadingDialog;
import com.compdfkit.tools.common.views.pdfview.CPDFViewCtrl;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.docseditor.pdfpageedit.CPDFPageEditDialogFragment;
import com.compdfkit.tools.forms.pdfproperties.pdfcombobox.CustomComboBoxWidgetImpl;
import com.compdfkit.tools.forms.pdfproperties.pdflistbox.CustomListBoxWidgetImpl;
import com.compdfkit.tools.forms.pdfproperties.pdfsign.CustomSignatureWidgetImpl;
import com.compdfkit.tools.viewer.contextmenu.CopyContextMenuView;
import com.compdfkit.tools.viewer.pdfdisplaysettings.CPDFDisplaySettingDialogFragment;
import com.compdfkit.tools.viewer.pdfinfo.CPDFDocumentInfoDialogFragment;

import java.io.File;

public class CBasicPDFFragment extends CPermissionFragment {

    protected CPDFConfiguration cpdfConfiguration;

    public int curEditMode = CPDFEditPage.LoadNone;

    protected Uri documentUri;

    protected String documentPath;

    protected void resetContextMenu(CPDFViewCtrl pdfView, CPreviewMode mode) {
        switch (mode) {
            case Viewer:
                pdfView.getCPdfReaderView().setContextMenuShowListener(
                        new CPDFContextMenuHelper.Builder()
                                .setSelectContentMenu(new CopyContextMenuView())
                                .setScreenShotContextMenu(new CScreenShotContextMenuView())
                                .create(pdfView));
                break;
            case Annotation:
                pdfView.getCPdfReaderView().setContextMenuShowListener(
                        new CPDFContextMenuHelper.Builder().defaultHelper()
                                .create(pdfView));
                break;
            case Edit:
                pdfView.getCPdfReaderView().setContextMenuShowListener(
                        new CPDFContextMenuHelper.Builder()
                                .setEditTextContentMenu(new CEditTextContextMenuView())
                                .setEditImageContentMenu(new CEditImageContextMenuView())
                                .setSearchReplaceContextMenu(new CSearchReplaceContextMenuView())
                                .setScreenShotContextMenu(new CScreenShotContextMenuView())
                                .create(pdfView));
                break;
            case Form:
                pdfView.getCPdfReaderView().setContextMenuShowListener(
                        new CPDFContextMenuHelper.Builder()
                                .defaultFormHelper()
                                .create(pdfView));
                break;
            case Signature:
                pdfView.getCPdfReaderView().setContextMenuShowListener(
                        new CPDFContextMenuHelper.Builder()
                                .setSignatureContextMenu(new CSignatureContextMenuView())
                                .setScreenShotContextMenu(new CScreenShotContextMenuView())
                                .create(pdfView));
                break;
            default:
                break;
        }
    }

    protected void registerAnnotHelper(CPDFViewCtrl pdfView) {
        pdfView.getCPdfReaderView().getAnnotImplRegistry().registAttachHelper(CPDFTextAnnotation.class, CPDFtextAnnotAttachHelper.class);
        pdfView.getCPdfReaderView().getAnnotImplRegistry().registImpl(CPDFTextAnnotation.class, CPDFtextAnnotImpl.class);
        pdfView.getCPdfReaderView().getAnnotImplRegistry().registAttachHelper(CPDFLinkAnnotation.class, CLinkAnnotAttachHelper.class);
    }

    protected void registerFormHelper(CPDFViewCtrl pdfView) {
        pdfView.getCPdfReaderView().getAnnotImplRegistry()
                //Register the CustomComboBoxWidgetImpl.class to implement a custom dropdown options popup.
                .registImpl(CPDFComboboxWidget.class, CustomComboBoxWidgetImpl.class)
                // Register the CustomListBoxWidgetImpl.class to implement a custom dropdown options popup.
                .registImpl(CPDFListboxWidget.class, CustomListBoxWidgetImpl.class)
                // Register the CustomSignatureWidgetImpl.class to implement a custom dropdown options popup.
                .registImpl(CPDFSignatureWidget.class, CustomSignatureWidgetImpl.class);
    }

    protected void showDisplaySettings(CPDFViewCtrl pdfView) {
        pdfView.exitEditMode();
        CPDFDisplaySettingDialogFragment displaySettingDialogFragment = CPDFDisplaySettingDialogFragment.newInstance();
        displaySettingDialogFragment.initWithPDFView(pdfView);
        displaySettingDialogFragment.show(getChildFragmentManager(), "displaySettingsDialog");
    }

    protected void showDocumentInfo(CPDFViewCtrl pdfView) {
        pdfView.exitEditMode();
        CPDFDocumentInfoDialogFragment infoDialogFragment = CPDFDocumentInfoDialogFragment.newInstance();
        infoDialogFragment.initWithPDFView(pdfView);
        infoDialogFragment.show(getChildFragmentManager(), "documentInfoDialogFragment");
    }

    protected void sharePDF(CPDFViewCtrl pdfView) {
        pdfView.savePDF((filePath, pdfUri) -> {
            if (null != documentUri) {
                CFileUtils.shareFile(getContext(), getString(R.string.tools_share_to), "application/pdf", documentUri);
            } else {
                CFileUtils.shareFile(getContext(), getString(R.string.tools_share_to), "application/pdf", new File(documentPath));
            }
        }, e -> {
            if (null != documentUri) {
                CFileUtils.shareFile(getContext(), getString(R.string.tools_share_to), "application/pdf", documentUri);
            } else {
                CFileUtils.shareFile(getContext(), getString(R.string.tools_share_to), "application/pdf", new File(documentPath));
            }
        });
    }

    public void showPageEdit(CPDFViewCtrl pdfView, boolean enterEdit, CPDFPageEditDialogFragment.OnBackLisener backListener) {
        curEditMode = pdfView.getCPdfReaderView().getLoadType();
        pdfView.exitEditMode();
        pdfView.getCPdfReaderView().getContextMenuShowListener().dismissContextMenu();
        CPDFPageEditDialogFragment pageEditDialogFragment = CPDFPageEditDialogFragment.newInstance();
        pageEditDialogFragment.initWithPDFView(pdfView);
        pageEditDialogFragment.setEnterEdit(enterEdit);
        pageEditDialogFragment.setOnBackListener(backListener);
        pageEditDialogFragment.show(getChildFragmentManager(), "pageEditDialogFragment");
    }

    private CLoadingDialog loadingDialog;

    protected void showLoadingDialog() {
        if (loadingDialog != null && loadingDialog.isVisible()) {
            loadingDialog.dismiss();
        }
        if (getContext() instanceof FragmentActivity) {
            loadingDialog = CLoadingDialog.newInstance();
            loadingDialog.show(((FragmentActivity) getContext()).getSupportFragmentManager(), "loadingDialog");
        }
    }

    protected void dismissLoadingDialog() {
        if (loadingDialog != null) {
            loadingDialog.dismiss();
        }
    }
}
